@@ -1373,13 +1373,13 @@ case 28:
 case 29:
 #line 442 "parser.y"
 {
-	if (yyvsp[-4].declspec) yywarningv (IDL_WARNING1,
-			    "Ignoring declspec for forward declaration `%s'",
-			    IDL_IDENT (yyvsp[-1].tree));
 	if (yyvsp[-3].hash_table) yywarningv (IDL_WARNING1,
 			    "Ignoring properties for forward declaration `%s'",
 			    IDL_IDENT (yyvsp[-1].tree));
 	yyval.tree = IDL_forward_dcl_new (yyvsp[-1].tree);
+	IDL_NODE_DECLSPEC (yyval.tree) = yyvsp[-4].declspec;
+	if (__IDL_inhibits > 0)
+		IDL_NODE_DECLSPEC (yyval.tree) |= IDLF_DECLSPEC_EXIST | IDLF_DECLSPEC_INHIBIT;
 ;
     break;}
 case 30:
