@@ -18,7 +18,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: util.c,v 1.74 1999/02/06 18:33:31 andrewtv Exp $
+    $Id: util.c,v 1.75 1999/02/12 22:27:24 andrewtv Exp $
 
 ***************************************************************************/
 #include <assert.h>
@@ -482,9 +482,13 @@ void yyerrorl (const char *s, int ofs)
 	int line = __IDL_cur_line - 1 + ofs;
 	gchar *filename = NULL;
 
-	if (__IDL_cur_filename)
+	if (__IDL_cur_filename) {
+#ifdef HAVE_CPP_PIPE_STDIN
+		filename = __IDL_cur_filename;
+#else
 		filename = g_basename (__IDL_cur_filename);
-	else
+#endif
+	} else
 		line = -1;
 
 	++__IDL_nerrors;
@@ -513,9 +517,13 @@ void yywarningl (int level, const char *s, int ofs)
 	if (__IDL_max_msg_level < level)
 		return;
 	
-	if (__IDL_cur_filename)
+	if (__IDL_cur_filename) {
+#ifdef HAVE_CPP_PIPE_STDIN
+		filename = __IDL_cur_filename;
+#else
 		filename = g_basename (__IDL_cur_filename);
-	else
+#endif
+	} else
 		line = -1;
 
 	++__IDL_nwarnings;
