@@ -30,6 +30,11 @@
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <signal.h>
+#include <syslog.h>
+
+#if defined(HAVE_TCPD_H) && defined(HAVE_HOSTS_ACCESS)
+#include <tcpd.h>
+#endif
 
 #if 0
 #include <malloc.h>
@@ -65,6 +70,7 @@ static void giop_connection_add_to_list         (GIOPConnection *cnx);
 static void giop_connection_remove_from_list    (GIOPConnection *cnx);
 
 static void iiop_init                           (void);
+static void iiop_connection_server_accept       (GIOPConnection *connection);
 static void iiop_connection_destroy   		(IIOPConnection *connection);
 static IIOPConnection *iiop_connection_new      (const char *host, gushort port);
 static IIOPConnection *iiop_connection_unix_new (const char *sockpath);
@@ -74,6 +80,10 @@ DEFINE_LOCK(giop_connection_list);
 GIOPConnectionList giop_connection_list;
 static GSList *iiop_unix_socket_list = NULL;
 
+#if defined(HAVE_HOSTS_ACCESS) && defined (HAVE_TCPD_H)
+static const char *argv0_val = NULL;
+#endif
+
 /*
  * giop_init
  *
@@ -89,11 +99,15 @@ static GSList *iiop_unix_socket_list = NULL;
  *		   to perform IIOP-specific initialization.
  */
 
-void giop_init(void)
+void giop_init(const char *argv0)
 {
   struct sigaction mypipe;
   g_assert(sizeof(GIOPMessageHeader) == 12);
 
+#if defined(HAVE_HOSTS_ACCESS) && defined (HAVE_TCPD_H)
+  argv0_val = g_strdup(g_basename(argv0)); /* For TCP wrappers */
+#endif
+
   memset(&mypipe, '\0', sizeof(mypipe));
   mypipe.sa_handler = SIG_IGN;
 
@@ -929,6 +943,74 @@ failed:
   return NULL;
 }
 
+/*
+ * iiop_connection_server_accept
+ *    Inputs: 'connection' - a server IIOPConnection.
+ *
+ *    Description: Performs accept(), TCP wrapper, access checking and related
+ *                 duties on a connection
+ */
+#if defined(HAVE_HOSTS_ACCESS) && defined(HAVE_TCPD_H)
+DEFINE_LOCK(tcp_wrappers_usage);
+
+int allow_severity, deny_severity;
+#endif
+static void
+iiop_connection_server_accept(GIOPConnection *connection)
+{
+  struct sockaddr sock;
+  int n;
+  int newfd;
+  GIOPConnection *newcnx;
+
+  n = sizeof(sock);
+
+  switch(IIOP_CONNECTION(connection)->icnxtype) {
+  case IIOP_IPV4: sock.sa_family = AF_INET; break;
+  case IIOP_USOCK: sock.sa_family = AF_UNIX; break;
+  case IIOP_IPV6:
+#ifdef HAVE_IPV6
+    sock.sa_family = AF_INET6;
+#endif
+    break;
+  }
+
+  newfd = accept(GIOP_CONNECTION_GET_FD(connection), &sock, &n);
+
+#if defined(HAVE_HOSTS_ACCESS) && defined(HAVE_TCPD_H)
+  /* tcp wrappers access checking */
+  switch(IIOP_CONNECTION(connection)->icnxtype) {
+  case IIOP_IPV4:
+    {
+      struct request_info request;
+
+      GET_LOCK(tcp_wrappers_usage);
+
+      request_init(&request, RQ_DAEMON, argv0_val, RQ_FILE, newfd, 0);
+
+      fromhost(&request);
+      if(!hosts_access(&request)) {
+	syslog(deny_severity, "[orbit] refused connect from %s", eval_client(&request));
+	close(newfd); newfd = -1;
+      } else
+	syslog(allow_severity, "[orbit] connect from %s", eval_client(&request));
+
+      RELEASE_LOCK(tcp_wrappers_usage);
+    }
+    break;
+  default:
+    /* No access controls for these transports */
+    break;
+  }
+#endif
+
+  if(newfd >= 0) {
+    newcnx = GIOP_CONNECTION(iiop_connection_from_fd(newfd,
+						     IIOP_CONNECTION(connection)));
+    GIOP_CONNECTION(newcnx)->orb_data = connection->orb_data;
+  }
+}
+
 /*
  * iiop_connection_destroy
  *
@@ -1035,29 +1117,9 @@ giop_main_next_message_2(gboolean blocking,
 	g_assert(!"connection has -ve fd!");
       }
 
-      if(connection->connection_class == GIOP_CONNECTION_SERVER) {
-	struct sockaddr sock;
-	int n;
-	int newfd;
-	GIOPConnection *newcnx;
-
-	n = sizeof(sock);
-
-	switch(IIOP_CONNECTION(connection)->icnxtype) {
-	case IIOP_IPV4: sock.sa_family = AF_INET; break;
-	case IIOP_USOCK: sock.sa_family = AF_UNIX; break;
-	case IIOP_IPV6:
-#ifdef HAVE_IPV6
- sock.sa_family = AF_INET6;
-#endif
-	  break;
-	}
-
-	newfd = accept(GIOP_CONNECTION_GET_FD(connection), &sock, &n);
-
-	newcnx = GIOP_CONNECTION(iiop_connection_from_fd(newfd, IIOP_CONNECTION(connection)));
-	GIOP_CONNECTION(newcnx)->orb_data = connection->orb_data;
-      } else
+      if(connection->connection_class == GIOP_CONNECTION_SERVER)
+	iiop_connection_server_accept(connection);
+      else
 	recv_buffer = giop_recv_message_buffer_use(connection);
 
       if(monitor && !monitor->is_valid) return NULL;
@@ -1076,29 +1138,7 @@ giop_main_handle_connection(GIOPConnection *connection)
   g_return_if_fail(connection->is_valid);
 
   if(connection->connection_class == GIOP_CONNECTION_SERVER) {
-    struct sockaddr sock;
-    int n;
-    int newfd;
-    GIOPConnection *newcnx;
-
-    n = sizeof(sock);
-
-    switch(IIOP_CONNECTION(connection)->icnxtype) {
-    case IIOP_IPV4: sock.sa_family = AF_INET; break;
-    case IIOP_USOCK: sock.sa_family = AF_UNIX; break;
-    case IIOP_IPV6:
-#ifdef HAVE_IPV6
-      sock.sa_family = AF_INET6;
-#endif
-      break;
-    }
-
-    newfd = accept(GIOP_CONNECTION_GET_FD(connection), &sock, &n);
-
-    newcnx = GIOP_CONNECTION(iiop_connection_from_fd(newfd,
-						     IIOP_CONNECTION(connection)));
-    GIOP_CONNECTION(newcnx)->orb_data = connection->orb_data;
-
+    iiop_connection_server_accept(connection);
     return;
   } else
     recv_buffer = giop_recv_message_buffer_use(connection);
