@@ -484,6 +484,17 @@ giop_message_buffer_append_mem(GIOPMessageBuffer *buffer,
   buffer->message_header.message_size += mem_region_length;
 }
 
+/* I think we need a WE_WANT_NEW_CRAPPY_BUGGY_CODE ifdef here - this
+   tiny routine seems to be horribly hard to get right.
+
+   Basically we have to paste the whole of 'mem_region' into our
+   memory chunks, possibly subdividing it up to fit it into multiple
+   1K chunks. Because we have to return the first paste point in case
+   the client wants to manipulate it afterwards, if mem_region_length
+   >= sizeof(CORBA_unsigned_long), we also have to guarantee that the
+   pasted stuff doesn't get divided on a finer boundary than
+   sizeof(CORBA_unsigned_long).
+*/
 gpointer
 giop_send_buffer_append_mem_indirect(GIOPSendBuffer *send_buffer,
 				     gconstpointer mem_region,
@@ -496,7 +507,7 @@ giop_send_buffer_append_mem_indirect(GIOPSendBuffer *send_buffer,
     thisblock_size = MIN(mem_region_length - offset,
 			 GIOP_INDIRECT_CHUNK_SIZE - send_buffer->indirect_used);
 
-    if(thisblock_size >= sizeof(CORBA_unsigned_long)
+    if((thisblock_size >= sizeof(CORBA_unsigned_long))
        || (mem_region_length - offset) < sizeof(CORBA_unsigned_long)) {
       if (!blockstart)
 	(guchar*)blockstart =
@@ -512,16 +523,14 @@ giop_send_buffer_append_mem_indirect(GIOPSendBuffer *send_buffer,
       send_buffer->indirect_used += thisblock_size;
     }
 
-    if(send_buffer->indirect_used >= GIOP_INDIRECT_CHUNK_SIZE
+    if(send_buffer->indirect_used >= (GIOP_INDIRECT_CHUNK_SIZE - sizeof(CORBA_unsigned_long))) {
+#ifdef I_CANT_FIGURE_OUT_WHAT_THIS_LOGIC_WAS_MEANT_TO_DO
        || (thisblock_size >= sizeof(CORBA_unsigned_long)
 	   && (mem_region_length - offset) > 0)) {
+#endif
 	  send_buffer->indirect_used = 0;
 	  send_buffer->indirect = g_chunk_new(gpointer,
 					      send_buffer->indirects);
-#ifdef DEBUG
-	  memset(send_buffer->indirect, '\xFE', GIOP_INDIRECT_CHUNK_SIZE);
-#endif
-
     }
   }
 
