@@ -15,13 +15,15 @@
 /* Input: main part of filename to write, namespace & tree from parser
    Output: (in files) header, skeletons, stubs, and common code.
 */
-#ifndef HAVE_INDENT
-#define INDENT "cat"
-#elif HAVE_INDENT_KR
-#define INDENT "indent -npro -kr -i8 -bad -fca -sc -sob"
+#ifdef HAVE_INDENT_KR
+#  define INDENT "indent -npro -kr -i8 -bad -fca -sc -sob"
 #else
-#define INDENT "indent -npro -bad -bap -bc -sob -br -ce -cli2 -npcs -di1 -psl -i3 -lp"
-#endif /* HAVE_INDENT */
+#  ifdef HAVE_INDENT
+#    define INDENT "indent -npro -bad -bap -bc -sob -br -ce -cli2 -npcs -di1 -psl -i3 -lp"
+#  else
+#    define INDENT "cat"
+#  endif
+#endif
 
 void
 orbit_IDL_tree_to_c(const char *base_filename, IDL_ns namespace, IDL_tree tree)
@@ -35,47 +37,59 @@ orbit_IDL_tree_to_c(const char *base_filename, IDL_ns namespace, IDL_tree tree)
   
   base_filename = g_basename(base_filename);
 
-  g_string_sprintf(curfile_name, INDENT " > %s.h", base_filename);
-  curfile = popen(curfile_name->str, "w");
-  if(!curfile) {
-    g_warning("Failed to open %s for writing: %s",
-	      curfile_name->str, g_strerror(errno));
-    return;
+  if(!disable_headers) {
+    g_string_sprintf(curfile_name, INDENT " > %s.h", base_filename);
+    curfile = popen(curfile_name->str, "w");
+    if(!curfile) {
+      g_warning("Failed to open %s for writing: %s",
+	        curfile_name->str, g_strerror(errno));
+      return;
+    }
+    orbit_cbe_write_header(curfile, namespace, tree);
+    fflush(curfile);
+    fclose(curfile);
   }
-  orbit_cbe_write_header(curfile, namespace, tree);
   g_string_sprintf(curfile_name, "%s.h", base_filename);
   hdrname = g_strdup(g_basename(curfile_name->str));
-  fclose(curfile);
 
-  g_string_sprintf(curfile_name, INDENT " > %s-common.c", base_filename);
-  curfile = popen(curfile_name->str, "w");
-  if(!curfile) {
-    g_warning("Failed to open %s for writing: %s",
-	      curfile_name->str, g_strerror(errno));
-    return;
+  if(!disable_common) {
+    g_string_sprintf(curfile_name, INDENT " > %s-common.c", base_filename);
+    curfile = popen(curfile_name->str, "w");
+    if(!curfile) {
+      g_warning("Failed to open %s for writing: %s",
+	        curfile_name->str, g_strerror(errno));
+      return;
+    }
+    orbit_cbe_write_common(curfile, namespace, tree, hdrname);
+    fflush(curfile);
+    fclose(curfile);
   }
-  orbit_cbe_write_common(curfile, namespace, tree, hdrname);
-  fclose(curfile);
 
-  g_string_sprintf(curfile_name, INDENT " > %s-stubs.c", base_filename);
-  curfile = popen(curfile_name->str, "w");
-  if(!curfile) {
-    g_warning("Failed to open %s for writing: %s",
-	      curfile_name->str, g_strerror(errno));
-    return;
+  if(!disable_stubs) {
+    g_string_sprintf(curfile_name, INDENT " > %s-stubs.c", base_filename);
+    curfile = popen(curfile_name->str, "w");
+    if(!curfile) {
+      g_warning("Failed to open %s for writing: %s",
+	        curfile_name->str, g_strerror(errno));
+      return;
+    }
+    orbit_cbe_write_stubs(curfile, namespace, tree, hdrname);
+    fflush(curfile);
+    fclose(curfile);
   }
-  orbit_cbe_write_stubs(curfile, namespace, tree, hdrname);
-  fclose(curfile);
 
-  g_string_sprintf(curfile_name, INDENT " > %s-skels.c", base_filename);
-  curfile = popen(curfile_name->str, "w");
-  if(!curfile) {
-    g_warning("Failed to open %s for writing: %s",
-	      curfile_name->str, g_strerror(errno));
-    return;
+  if(!disable_skels) {
+    g_string_sprintf(curfile_name, INDENT " > %s-skels.c", base_filename);
+    curfile = popen(curfile_name->str, "w");
+    if(!curfile) {
+      g_warning("Failed to open %s for writing: %s",
+	        curfile_name->str, g_strerror(errno));
+      return;
+    }
+    orbit_cbe_write_skeletons(curfile, namespace, tree, hdrname);
+    fflush(curfile);
+    fclose(curfile);
   }
-  orbit_cbe_write_skeletons(curfile, namespace, tree, hdrname);
-  fclose(curfile);
 
   if(enable_skeleton_impl) {
     g_string_sprintf(curfile_name, INDENT " > %s-impl.c", base_filename);
@@ -86,6 +100,7 @@ orbit_IDL_tree_to_c(const char *base_filename, IDL_ns namespace, IDL_tree tree)
       return;
     }
     orbit_cbe_write_skelimpl(curfile, namespace, tree, hdrname);
+    fflush(curfile);
     fclose(curfile);
   }
 
@@ -810,6 +825,7 @@ orbit_cbe_param_numptrs(IDL_tree param, IDL_ParamRole role)
 	retval++;
       break;
     default:
+      break;
     }
 
   g_assert(typeoffsets[IDL_NODE_TYPE(param)] >= 0);
