@@ -202,7 +202,7 @@ static void cbe_output_demarshal_type_string(CBEDemarshalInfo *mi)
     fprintf(mi->of, "    ((char *)GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur) += len;\n");
   } else {
     /* We cheat, and use the 0's in the length as the zero-byte of the zero-length string */
-    fprintf(mi->of, "    %s = len?(GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur):(GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur - sizeof(CORBA_unsigned_long));", mi->param_name);
+    fprintf(mi->of, "    %s = len?(GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur):(((char *)GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur) - sizeof(CORBA_unsigned_long));", mi->param_name);
     fprintf(mi->of, "    ((char *)GIOP_RECV_BUFFER(_ORBIT_recv_buffer)->cur) += len;\n");
   }
 
