@@ -120,7 +120,7 @@ ORBit_free(gpointer mem, CORBA_boolean free_strings)
 		gpointer my_data;
 
 		if((gpointer)block->free == (gpointer)ORBit_free_via_TypeCode)
-			my_data = ((char *)block) - sizeof(CORBA_TypeCode);
+			my_data = ((guchar *)block) - sizeof(CORBA_TypeCode);
 		else
 			my_data = NULL;
 
@@ -133,7 +133,7 @@ ORBit_free(gpointer mem, CORBA_boolean free_strings)
 			x = block->free(x, my_data, free_strings);
 
 		if((gpointer)block->free == (gpointer)ORBit_free_via_TypeCode)
-			((char *)block) -= sizeof(CORBA_TypeCode);
+			((guchar *)block) -= sizeof(CORBA_TypeCode);
 		g_free(block);
 	} else
 		g_free(block);
@@ -147,7 +147,7 @@ ORBit_free_via_TypeCode(gpointer mem, gpointer tcp, gboolean free_strings)
 {
 	CORBA_TypeCode tc = *(CORBA_TypeCode *)tcp, subtc;
 	int i, n;
-	gpointer retval = NULL;
+	guchar *retval = NULL;
 
 	switch(tc->kind) {
 	case CORBA_tk_any:
@@ -155,7 +155,7 @@ ORBit_free_via_TypeCode(gpointer mem, gpointer tcp, gboolean free_strings)
 			CORBA_any *anyval = mem;
 			if(anyval->_release)
 				CORBA_free(anyval->_value);
-			retval = anyval + 1;
+			retval = (guchar *)(anyval + 1);
 		}
 		break;
 	case CORBA_tk_TypeCode:
@@ -163,7 +163,7 @@ ORBit_free_via_TypeCode(gpointer mem, gpointer tcp, gboolean free_strings)
 		{
 			CORBA_Object_release(*(CORBA_Object *)mem, NULL);
 
-			retval = mem + sizeof(CORBA_Object);
+			retval = (guchar *)mem + sizeof(CORBA_Object);
 		}
 		break;
 	case CORBA_tk_Principal:
@@ -171,7 +171,7 @@ ORBit_free_via_TypeCode(gpointer mem, gpointer tcp, gboolean free_strings)
 			CORBA_Principal *pval = mem;
 			if(pval->_release)
 				CORBA_free(pval->_buffer);
-			retval = pval + 1;
+			retval = (guchar *)(pval + 1);
 		}
 		break;
 	case CORBA_tk_except:
@@ -186,7 +186,7 @@ ORBit_free_via_TypeCode(gpointer mem, gpointer tcp, gboolean free_strings)
 		break;
 	case CORBA_tk_union:
 		subtc = (CORBA_TypeCode)CORBA_Object_duplicate((CORBA_Object)ORBit_get_union_tag(tc, mem), NULL);
-		mem += ORBit_gather_alloc_info(tc->discriminator);
+		(guchar *) mem += ORBit_gather_alloc_info(tc->discriminator);
 		n = 1;
 		for(i = 0; i < tc->sub_parts; i++) {
 			n = MAX(n, ORBit_find_alignment(tc->subtypes[i]));
@@ -198,7 +198,7 @@ ORBit_free_via_TypeCode(gpointer mem, gpointer tcp, gboolean free_strings)
 	case CORBA_tk_string:
 		if(free_strings)
 			CORBA_free(*(char **)mem);
-		retval = mem + sizeof(char *);
+		retval = (guchar *)mem + sizeof(char *);
 		break;
 	case CORBA_tk_sequence:
 		{
@@ -206,7 +206,7 @@ ORBit_free_via_TypeCode(gpointer mem, gpointer tcp, gboolean free_strings)
 			if(pval->_release)
 				CORBA_free(pval->_buffer);
 
-			retval = mem + sizeof(CORBA_sequence_octet);
+			retval = (guchar *)mem + sizeof(CORBA_sequence_octet);
 		}
 		break;
 	case CORBA_tk_array:
@@ -222,11 +222,11 @@ ORBit_free_via_TypeCode(gpointer mem, gpointer tcp, gboolean free_strings)
 		retval = ORBit_free_via_TypeCode(mem, &subtc, free_strings);
 		break;
 	default:
-		retval = ((char *)mem) + ORBit_gather_alloc_info(tc);
+		retval = ((guchar *)mem) + ORBit_gather_alloc_info(tc);
 		break;
 	}
 
 	CORBA_Object_release((CORBA_Object)tc, NULL);
 
-	return retval;
+	return (gpointer)retval;
 }
