@@ -89,28 +89,28 @@ typedef enum {
 #define TC_CORBA_string TC_string
 #define TC_CORBA_wstring TC_wstring
 
-extern const CORBA_TypeCode TC_null;
-extern const CORBA_TypeCode TC_void;
-extern const CORBA_TypeCode TC_short;
-extern const CORBA_TypeCode TC_long;
-extern const CORBA_TypeCode TC_longlong;
-extern const CORBA_TypeCode TC_ushort;
-extern const CORBA_TypeCode TC_ulong;
-extern const CORBA_TypeCode TC_ulonglong;
-extern const CORBA_TypeCode TC_float;
-extern const CORBA_TypeCode TC_double;
-extern const CORBA_TypeCode TC_longdouble;
-extern const CORBA_TypeCode TC_boolean;
-extern const CORBA_TypeCode TC_char;
-extern const CORBA_TypeCode TC_wchar;
-extern const CORBA_TypeCode TC_octet;
-extern const CORBA_TypeCode TC_any;
-extern const CORBA_TypeCode TC_TypeCode;
-extern const CORBA_TypeCode TC_Principal;
-extern const CORBA_TypeCode TC_Object;
-extern const CORBA_TypeCode TC_string;
-extern const CORBA_TypeCode TC_wstring;
-extern const CORBA_TypeCode TC_CORBA_NamedValue;
+#define TC_null (&TC_null_struct)
+#define TC_void (&TC_void_struct)
+#define TC_short (&TC_short_struct)
+#define TC_long (&TC_long_struct)
+#define TC_longlong (&TC_longlong_struct)
+#define TC_ushort (&TC_ushort_struct)
+#define TC_ulong (&TC_ulong_struct)
+#define TC_ulonglong (&TC_ulonglong_struct)
+#define TC_float (&TC_float_struct)
+#define TC_double (&TC_double_struct)
+#define TC_longdouble (&TC_longdouble_struct)
+#define TC_boolean (&TC_boolean_struct)
+#define TC_char (&TC_char_struct)
+#define TC_wchar (&TC_wchar_struct)
+#define TC_octet (&TC_octet_struct)
+#define TC_any (&TC_any_struct)
+#define TC_TypeCode (&TC_TypeCode_struct)
+#define TC_Principal (&TC_Principal_struct)
+#define TC_Object (&TC_Object_struct)
+#define TC_string (&TC_string_struct)
+#define TC_wstring (&TC_wstring_struct)
+#define TC_CORBA_NamedValue (&TC_CORBA_NamedValue_struct)
 
 #define TC_CORBA_short_struct TC_short_struct
 #define TC_CORBA_long_struct TC_long_struct
