@@ -177,7 +177,7 @@ ORBit_RootObject_set_interface(ORBit_RootObject obj,
 	obj->interface = epv;
 }
 
-#define GET_ATOM(x) ({ GIOP_RECV_BUFFER(recv_buffer)->decoder(&x, (GIOP_RECV_BUFFER(recv_buffer)->cur), sizeof(x)); GIOP_RECV_BUFFER(recv_buffer)->cur += sizeof(x); })
+#define GET_ATOM(x) G_STMT_START{ GIOP_RECV_BUFFER(recv_buffer)->decoder(&x, (GIOP_RECV_BUFFER(recv_buffer)->cur), sizeof(x)); ((guchar *)GIOP_RECV_BUFFER(recv_buffer)->cur) += sizeof(x); }G_STMT_END
 #define ALIGNFOR(x) recv_buffer->cur = ALIGN_ADDRESS(recv_buffer->cur, sizeof(x))
 
 CORBA_Object
@@ -208,8 +208,10 @@ ORBit_create_object_with_info(GSList *profiles,
 	refcheck.profile_list = profiles;
 
 	new = g_hash_table_lookup(orb->objrefs, &refcheck);
-	if(new)
+	if(new) {
+		ORBit_delete_profiles(profiles);
 		return CORBA_Object_duplicate(new, ev);
+	}
 
 	new = ORBit_CORBA_Object_new(ev);
 	new->connection = NULL;
@@ -231,17 +233,18 @@ ORBit_demarshal_profile(GIOPRecvBuffer *recv_buffer, IOP_ProfileId profile_id)
 	ORBit_Object_info *object_info;
 	CORBA_unsigned_long subpart_len;
 	CORBA_octet o;
-	CDR_Codec *codec=NULL;
+	CDR_Codec codec_d;
+	CDR_Codec *codec=&codec_d;
 
 	object_info = g_new0(ORBit_Object_info, 1);
 
 	switch(profile_id) {
 	case IOP_TAG_INTERNET_IOP:
 		GET_ATOM(subpart_len); /* The length of the embedded sequence */
-		codec = CDR_codec_init();
+		CDR_codec_init_static(codec);
 		codec->buffer = recv_buffer->cur;
 		codec->release_buffer = CORBA_FALSE;
-		recv_buffer->cur += subpart_len;
+		((guchar *)recv_buffer->cur) += subpart_len;
 
 		codec->readonly = CORBA_TRUE;
 		codec->host_endian = codec->data_endian = FLAG_ENDIANNESS;
@@ -271,7 +274,6 @@ ORBit_demarshal_profile(GIOPRecvBuffer *recv_buffer, IOP_ProfileId profile_id)
 
 		ORBit_set_object_key(object_info);
 
-		CDR_codec_free(codec);
 		return(object_info);
 		break;
 
@@ -279,16 +281,16 @@ ORBit_demarshal_profile(GIOPRecvBuffer *recv_buffer, IOP_ProfileId profile_id)
 		GET_ATOM(subpart_len);
 		g_warning("IOP_TAG_MULTIPLE_COMPONENTS decoding needs finishing");
 		object_info->profile_type = IOP_TAG_MULTIPLE_COMPONENTS;
-		recv_buffer->cur += subpart_len;
+		((guchar *)recv_buffer->cur) += subpart_len;
 		return(object_info);
 		break;
 
 	case IOP_TAG_ORBIT_SPECIFIC:
 		GET_ATOM(subpart_len);
-		codec = CDR_codec_init();
+		CDR_codec_init_static(codec);
 		codec->buffer = recv_buffer->cur;
 		codec->release_buffer = CORBA_FALSE;
-		recv_buffer->cur += subpart_len;
+		((guchar *)recv_buffer->cur) += subpart_len;
 
 		codec->readonly = CORBA_TRUE;
 		codec->host_endian = codec->data_endian = FLAG_ENDIANNESS;
@@ -317,16 +319,14 @@ ORBit_demarshal_profile(GIOPRecvBuffer *recv_buffer, IOP_ProfileId profile_id)
 
 		ORBit_set_object_key(object_info);
 
-		CDR_codec_free(codec);
 		return(object_info);
 		break;
 	}
+
 error_exit:
 	g_message("demarshal_profile(): IIOP major is %d",
 		  object_info->iiop_major);
 	g_free(object_info);
-	if(codec!=NULL)
-		CDR_codec_free(codec);
 
 	return(NULL);
 }
@@ -335,7 +335,6 @@ GSList *ORBit_demarshal_IOR(GIOPRecvBuffer *recv_buffer)
 {
 	GSList *profiles=NULL;
 	ORBit_Object_info *object_info;
-	CORBA_char *type_id;
 	CORBA_unsigned_long len, seq_len;
 	IOP_ProfileId profile_id;
 	int i;
@@ -347,8 +346,7 @@ GSList *ORBit_demarshal_IOR(GIOPRecvBuffer *recv_buffer)
 	if(len == 0)
 		return(NULL);
 
-	type_id = g_strdup(recv_buffer->cur);
-	recv_buffer->cur += len;
+	((guchar *)recv_buffer->cur) += len;
 
 	/* Decode the sequence<TaggedProfile> */
 	ALIGNFOR(CORBA_unsigned_long);
@@ -392,8 +390,8 @@ ORBit_demarshal_object(GIOPRecvBuffer *recv_buffer, CORBA_ORB orb)
 	if(len == 0)
 		return CORBA_OBJECT_NIL;
 
-	type_id = g_strdup(recv_buffer->cur);
-	recv_buffer->cur += len;
+	type_id = recv_buffer->cur;
+	((guchar *)recv_buffer->cur) += len;
 
 	/* Decode the sequence<TaggedProfile> */
 	ALIGNFOR(CORBA_unsigned_long);
@@ -412,8 +410,6 @@ ORBit_demarshal_object(GIOPRecvBuffer *recv_buffer, CORBA_ORB orb)
 
 	retval = ORBit_create_object_with_info(profiles, type_id, orb, &ev);
 
-	g_free(type_id);
-
 	return retval;
 
  error_exit:
@@ -429,8 +425,10 @@ static void ORBit_marshal_profile(gpointer item, gpointer data)
 	GIOPSendBuffer *send_buffer = (GIOPSendBuffer *)data;
 	static const CORBA_unsigned_long ioptag = IOP_TAG_INTERNET_IOP,
 		orbittag = IOP_TAG_ORBIT_SPECIFIC;
-	CDR_Codec *codec;
+	CDR_Codec codec_d;
+	CDR_Codec *codec = &codec_d;
 	CORBA_unsigned_long len;
+	char codecbuf[2048];
 	static const CORBA_octet oc_endian = FLAG_ENDIANNESS;
 	static const CORBA_octet iiopversion[] = {1,0};
 
@@ -441,9 +439,10 @@ static void ORBit_marshal_profile(gpointer item, gpointer data)
 		giop_message_buffer_append_mem(GIOP_MESSAGE_BUFFER(send_buffer),
 					       &ioptag, sizeof(ioptag));
 
-		codec = CDR_codec_init();
-		codec->buffer = g_malloc(2048);
+		CDR_codec_init_static(codec);
+		codec->buffer = codecbuf;
 		codec->buf_len = 2048;
+		codec->release_buffer = CORBA_FALSE;
 		codec->readonly = CORBA_FALSE;
 		codec->data_endian = codec->host_endian = FLAG_ENDIANNESS;
 		CDR_put_octet(codec, oc_endian);
@@ -459,12 +458,12 @@ static void ORBit_marshal_profile(gpointer item, gpointer data)
 						       &len, sizeof(len));
 		giop_send_buffer_append_mem_indirect(send_buffer,
 						     codec->buffer, codec->wptr);
-		CDR_codec_free(codec);
 	} else if(info->profile_type==IOP_TAG_ORBIT_SPECIFIC) {
 		giop_message_buffer_append_mem_a(GIOP_MESSAGE_BUFFER(send_buffer),
 						 &orbittag, sizeof(orbittag));
-		codec = CDR_codec_init();
-		codec->buffer = g_malloc(2048);
+		CDR_codec_init_static(codec);
+		codec->buffer = codecbuf;
+		codec->release_buffer = CORBA_FALSE;
 		codec->buf_len = 2048;
 		codec->readonly = CORBA_FALSE;
 		codec->data_endian = codec->host_endian = FLAG_ENDIANNESS;
@@ -480,7 +479,6 @@ static void ORBit_marshal_profile(gpointer item, gpointer data)
 						       &len, sizeof(len));
 		giop_send_buffer_append_mem_indirect(send_buffer,
 						     codec->buffer, codec->wptr);
-		CDR_codec_free(codec);
 	} else {
 		g_warning("ORBit_marshal_profile ask to marshal type %d\n", info->profile_type);
 	}
@@ -517,9 +515,6 @@ static void ORBit_test_profile(gpointer item, gpointer data)
 	ORBit_Object_info *info = (ORBit_Object_info *)item;
 	CORBA_Object obj = (CORBA_Object)data;
 
-	g_assert(info);
-	g_assert(obj);
-
 	if(obj->active_profile != NULL)
 		return;		/* we already have a good profile */
 
@@ -547,9 +542,6 @@ static void ORBit_activate_profile(gpointer item, gpointer data)
 	ORBit_Object_info *info = (ORBit_Object_info *)item;
 	CORBA_Object obj = (CORBA_Object)data;
 
-	g_assert(info);
-	g_assert(obj);
-
 	if(obj->active_profile != NULL)
 		return;		/* we already have a good profile */
 
