@@ -43,7 +43,8 @@ typedef enum {
 	ORBIT_PSEUDO_POAMANAGER,
 	ORBIT_PSEUDO_POLICY,
 	ORBIT_PSEUDO_TYPECODE,
-	ORBIT_PSEUDO_REQUEST
+	ORBIT_PSEUDO_REQUEST,
+	ORBIT_PSEUDO_SERVERREQUEST
 } ORBit_PseudoObject_type;
 typedef struct ORBit_PseudoObject_struct *ORBit_PseudoObject;
 
@@ -68,6 +69,7 @@ typedef struct {
 } TAG_ORBIT_SPECIFIC_info;
 
 typedef struct {
+	int fill_me_in;
 } TAG_MULTIPLE_COMPONENTS_info;
 
 typedef struct {
